use pulldown_cmark::{Parser, Options, html};
use std::path::Path;

/// Fix a name to be used as a key
///
/// This is equivalent to the tools.NameFix function in the Go implementation.
/// It normalizes the name by converting to lowercase, replacing spaces with hyphens, etc.
///
/// # Arguments
///
/// * `name` - The name to fix
///
/// # Returns
///
/// The fixed name
pub fn name_fix(name: &str) -> String {
    // Convert to lowercase
    let mut result = name.to_lowercase();
    
    // Replace spaces with hyphens
    result = result.replace(' ', "-");
    
    // Remove special characters
    result = result.chars()
        .filter(|c| c.is_alphanumeric() || *c == '-' || *c == '.')
        .collect();
    
    result
}

/// Convert markdown to HTML
///
/// # Arguments
///
/// * `markdown` - The markdown content to convert
///
/// # Returns
///
/// The HTML content
pub fn markdown_to_html(markdown: &str) -> String {
    let mut options = Options::empty();
    options.insert(Options::ENABLE_TABLES);
    options.insert(Options::ENABLE_FOOTNOTES);
    options.insert(Options::ENABLE_STRIKETHROUGH);
    
    let parser = Parser::new_ext(markdown, options);
    let mut html_output = String::new();
    html::push_html(&mut html_output, parser);
    
    html_output
}

/// Trim spaces and quotes from a string
///
/// # Arguments
///
/// * `s` - The string to trim
///
/// # Returns
///
/// The trimmed string
pub fn trim_spaces_and_quotes(s: &str) -> String {
    let mut result = s.trim().to_string();
    
    // Remove surrounding quotes
    if (result.starts_with('\'') && result.ends_with('\'')) || 
       (result.starts_with('"') && result.ends_with('"')) {
        result = result[1..result.len()-1].to_string();
    }
    
    result
}

/// Ensure a string has a .md extension
///
/// # Arguments
///
/// * `name` - The name to check
///
/// # Returns
///
/// The name with a .md extension
pub fn ensure_md_extension(name: &str) -> String {
    if !name.ends_with(".md") {
        format!("{}.md", name)
    } else {
        name.to_string()
    }
}

/// Get the file extension from a path
///
/// # Arguments
///
/// * `path` - The path to check
///
/// # Returns
///
/// The file extension or an empty string
pub fn get_extension(path: &str) -> String {
    Path::new(path)
        .extension()
        .and_then(|ext| ext.to_str())
        .unwrap_or("")
        .to_string()
}